/*-------------------------------------------------------------------------
	Copyright (C) 2008-20013 American Megatrends Inc.
	
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
-------------------------------------------------------------------------*/
#ifndef	_I2C_ARBITRATION_H
#define	_I2C_ARBITRATION_H

#include <linux/i2c.h>

typedef enum {

	STATUS_SUCCESS = 0,
	STATUS_INVALID_PARAM = -1,
	STATUS_INACCESSIBLE = -2,
	STATUS_OPER_INCOMPLETE = -3,
	STATUS_FAILURE = -4
		
} dev_status_e;

#define NO_ERROR                                0x00
#define CLEARABLE_ERROR_MASK                    0x1f

 /* PARALLEL_BUS_MLBR */
#define MLBR_LOC_BUS_NUM						0x07
#define MLBR_LOC_BUS_REQ						0x10
#define MLBR_LOC_BUS_ACT						0x20
#define MLBR_LOC_ACC_TYPE						0x40

/* PARALLEL_BUS_MTRR */
#define MTRR_REST_SRT_TIMER						0x01

/* PARALLEL_BUS_MRBR */
#define MRBR_RM_BUS_NUM							0x03
#define MRBR_RM_BUS_GRT							0x10
#define MRBR_RM_BUS_ACT							0x20
#define MRBR_RM_ACC_TYPE						0x40

/* PARALLEL_BUS_MLER */
#define MLER_LOC_REL_SHORT_TMO					0x01
#define MLER_LOC_REL_LONG_TMO					0x02
#define MLER_RM_GRT_TMO							0x04
#define MLER_RM_REL_TMO							0x08
#define MLER_LOC_SER_EXCH_ERR					0x10
#define MLER_LOC_INV_SER_EXCH_BUS				0x20
#define MLER_LOC_BUF_N_READY					0x40

/* PARALLEL_BUS_MTRR */
#define MTRR_START_STIMER						0x01

/* PARALLEL_BUS_MRER */
#define MRER_RM_REL_SHORT_TMO					0x01
#define MRER_RM_REL_LONG_TMO					0x02
#define MRER_LOC_GRT_TMO						0x04
#define MRER_LOC_REL_TMO						0x08
#define MRER_RM_SER_EXCH_ERR					0x10
#define MRER_RM_INV_SER_EXCH_BUS				0x20
#define MRER_RM_BUF_N_READY						0x40

#define SUPFPGA_BASE             0xB6000000
#define SUPFPGA_MCSR_REG         0x1002
#define SUPFPGA_MLBR_REG         0x1006
#define SUPFPGA_MTRR_REG         0x100A
#define SUPFPGA_MRBR_REG         0x100E
#define SUPFPGA_MLER_REG         0x1012
#define SUPFPGA_MRER_REG         0x1016
#define SUPFPGA_MSTR_REG         0x1022
#define SUPFPGA_MLTR_REG         0x1026

/* for support fpga debugging */
#define SUPFPGA_DEBUG0_REG       0xC002
#define SUPFPGA_DEBUG1_REG       0xC006
#define SUPFPGA_DEBUG2_REG       0xC00A
#define SUPFPGA_DEBUG3_REG       0xC00E
#define SUPFPGA_DEBUG4_REG       0xC012
#define SUPFPGA_DEBUG5_REG       0xC016

#define READ_SUPFPGA_MMAP_REG(reg)        (*((u16 *)(SUPFPGA_BASE + reg)))
#define WRITE_SUPFPGA_MMAP_REG(reg, val)  *((u16 *)(SUPFPGA_BASE + reg)) = (uint16_t)val

#define RESTART_SHORTTIMER

#define RESTART_TIMER_VALUE	         0x0001
#define RESTART_SHORT_TIMER(val)                   \
do {                                               \
    u8 mlbr;                                       \
    supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &mlbr); \
    if (mlbr & MLBR_LOC_BUS_ACT) {                 \
        fpga_start_short_timer();                  \
    }                                              \
} while(0);

extern dev_status_e supfpga_mm_reg8_read(u16 reg, u8 *read_buf);
extern dev_status_e supfpga_mm_reg8_write(u16 reg, u8 write_buf);

extern int i2c_request_shared_bus(u8 segnum);
extern int i2c_grant_shared_bus(u8 segnum);
extern int i2c_release_shared_bus(u8 segnum);

extern int fpga_start_short_timer(void);
extern int fpga_set_local_short_timeout(u8 timeout);
extern int fpga_set_local_long_timeout(u8 timeout);

#endif

