diff -Naur uboot.old/drivers/i2c/Makefile uboot/drivers/i2c/Makefile
--- uboot.old/drivers/i2c/Makefile	1969-12-31 19:00:00.000000000 -0500
+++ uboot/drivers/i2c/Makefile	2008-10-16 10:46:50.000000000 -0400
@@ -0,0 +1,47 @@
+#
+# (C) Copyright 2003
+# Wolfgang Denk, DENX Software Engineering, wd@denx.de.
+#
+# See file CREDITS for list of people who contributed to this
+# project.
+#
+# This program is free software; you can redistribute it and/or
+# modify it under the terms of the GNU General Public License as
+# published by the Free Software Foundation; either version 2 of
+# the License, or (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston,
+# MA 02111-1307 USA
+#
+#
+include $(TOPDIR)/config.mk
+
+LIB := libi2c.a
+
+OBJS := vsc452_i2c.o
+
+EXTRA_CFLAGS += -I. 
+
+CFLAGS += $(EXTRA_CFLAGS)
+
+
+all:	$(LIB)
+
+$(LIB):	$(OBJS)
+	$(AR) crv $@ $(OBJS)
+
+#########################################################################
+
+.depend:	Makefile $(OBJS:.o=.c)
+		$(CC) -M $(CFLAGS) $(OBJS:.o=.c) > $@
+
+sinclude .depend
+
+#########################################################################
diff -Naur uboot.old/drivers/i2c/vsc452_i2c.c uboot/drivers/i2c/vsc452_i2c.c
--- uboot.old/drivers/i2c/vsc452_i2c.c	1969-12-31 19:00:00.000000000 -0500
+++ uboot/drivers/i2c/vsc452_i2c.c	2013-10-14 14:12:05.801385000 -0400
@@ -0,0 +1,965 @@
+/*
+ * $Id$
+ *
+ * Copyright 2008, Network Appliance Inc.
+ * Author: Jason McMullan <mcmullan@netapp.com>
+ *
+ */
+
+#include <common.h>
+#include <i2c.h>
+#include <asm/io.h>
+
+#include "vsc452/vsc452_i2c.h"
+
+#define I2C_TIMEOUT	200	/* 200ms */
+
+#define WAIT_UNTIL(condition, timeout_ms) \
+	do { \
+		int time_left = timeout_ms * 1000; \
+		while (time_left > 0 && !(condition)) { \
+			udelay((time_left > 100) ? 100 : time_left); \
+			time_left -= (time_left > 100)  ? 100 : time_left; \
+		}; \
+	} while (0); if (!(condition))
+
+int i2c_bus;
+int shared_bus_seg=0;
+
+static struct vsc452_i2c_bus {
+	int speed;	/* offset into the speed map */
+	uint32_t base;	/* Physical address */
+} i2c[VSC452_I2C_INTERFACES];
+
+static const struct {
+	unsigned int speed;
+	unsigned int select;
+} i2c_speed_map[] = {
+	{ .speed =   78000, .select = SID_CMS_CLK_78KHZ },
+	{ .speed =   98000, .select = SID_CMS_CLK_98KHZ },
+	{ .speed =  284000, .select = SID_CMS_CLK_284KHZ },
+	{ .speed =  340000, .select = SID_CMS_CLK_340KHZ },
+	{ .speed =  347000, .select = SID_CMS_CLK_347KHZ },
+	{ .speed =  417000, .select = SID_CMS_CLK_417KHZ },
+	{ .speed =  852000, .select = SID_CMS_CLK_852KHZ },
+	{ .speed = 1042000, .select = SID_CMS_CLK_1042KHZ },
+};
+
+static inline uint32_t read32(uint32_t base)
+{
+	return *(volatile uint32_t *)(KSEG1ADDR(base));
+}
+
+static inline void write32(uint32_t base, uint32_t val)
+{
+	*(volatile uint32_t *)(KSEG1ADDR(base)) = val;
+}
+
+static inline uint32_t iread(int reg)
+{
+	return read32(i2c[i2c_bus].base + reg);
+}
+
+static inline void iwrite(int reg, uint32_t val)
+{
+	write32(i2c[i2c_bus].base + reg, val);
+}
+
+int i2c_set_bus_speed (unsigned int speed)
+{
+        int i;
+
+        for (i = 0; i < ARRAY_SIZE(i2c_speed_map); i++) {
+                if (speed <= i2c_speed_map[i].speed) {
+                        break;
+                }
+        }
+
+        /* Too fast? Then select the fastest possible.
+	 */
+        if (i == ARRAY_SIZE(i2c_speed_map)) {
+                i--;
+        }
+
+        i2c[i2c_bus].speed = i;
+        /* Set the speed, do not snoop the bus */
+        iwrite(SIC, SIC_PIN | SIC_ES_CMS);
+        iwrite(SID, i2c_speed_map[i2c[i2c_bus].speed].select);
+
+        /* Return to data i/o mode */
+        iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_ACK);
+
+        return 0;
+}
+
+int i2c_set_bus_num (unsigned int bus)
+{
+	int i;
+
+	if (bus >= VSC452_I2C_INTERFACES) {
+		return -1;
+	}
+
+	i2c_bus = bus;
+	i2c[i2c_bus].base = I2C_BASE(i2c_bus);
+
+	/* if shared bus */
+	if (i2c_bus == 2) {
+
+		/* request shared bus */
+		if (i2c_request_shared_bus(shared_bus_seg) != STATUS_SUCCESS) {
+			return -1;
+		}	
+		
+		/* wait for bus grant and if bus granted */
+		if (i2c_grant_shared_bus(shared_bus_seg) != STATUS_SUCCESS) {
+			return -1;
+		}
+		
+		/* start short timer */
+		fpga_start_short_timer();
+	}
+		
+	/* Reset the controller */
+	iwrite(SIBC, SIBC_SISR);
+	iwrite(SIBC, SIBC_SCC | SIBC_SDC);
+
+	/* Wiggle out any data on the line */
+	if ((iread(SIBS) & SIBS_SDS) == 0) {
+		for (i = 0; i < 8 * 16; i++) {
+			iwrite(SIBC,        0 | SIBC_SDC);
+			udelay(100);
+			iwrite(SIBC, SIBC_SCC | SIBC_SDC);
+			udelay(100);
+		}
+	}
+
+	/* Master only */
+	iwrite(SIC, SIC_PIN | SIC_ES_SCA);
+	iwrite(SID, SID_SCA_ADDR(0x7F));
+
+	/* Select the bus speed */
+	i2c_set_bus_speed(98000); /* use only 98khz i2c bus speed */
+
+	/* if shared bus */
+	if (i2c_bus == 2) {
+		/* release shared bus */
+		i2c_release_shared_bus(shared_bus_seg);
+	}
+	
+	return 0;
+}
+
+unsigned int i2c_get_bus_num (void)
+{
+	return i2c_bus;
+}
+
+int i2c_set_shared_bus_seg (unsigned int seg)
+{
+	if (seg > MAX_SEG_VAL) {
+		return -1;
+	}
+
+	shared_bus_seg = seg;
+
+	return 0;
+}
+
+unsigned int i2c_get_shared_bus_seg (void)
+{
+	return shared_bus_seg;
+}
+
+
+unsigned int i2c_get_bus_speed (void )
+{
+	return i2c_speed_map[i2c[i2c_bus].speed].speed;
+}
+
+
+#define I2C_STAT(tmp) \
+	do { \
+		debug("Status: 0x%08x (", tmp); \
+		if ((tmp & SIC_PIN) == SIC_PIN) debug("PIN "); \
+		if ((tmp & SIC_SSN) == SIC_SSN) debug("SSN "); \
+		if ((tmp & SIC_STS) == SIC_STS) debug("STS "); \
+		if ((tmp & SIC_BER) == SIC_BER) debug("BER "); \
+		if ((tmp & SIC_LRB) == SIC_LRB) debug("LRB "); \
+		if ((tmp & SIC_AAS) == SIC_AAS) debug("AAS "); \
+		if ((tmp & SIC_LAB) == SIC_LAB) debug("LAB "); \
+		if ((tmp & SIC_BBN) == SIC_BBN) debug("BBN "); \
+		debug(")\n"); \
+	} while (0)
+
+#define I2C_ABORT() do { \
+	i2c_set_bus_num(i2c_bus); \
+} while (0)
+
+
+int i2c_probe(uchar chip)
+{
+	uchar c[2];
+
+	return i2c_read(chip, 0, 0, &c[0], 2);
+}
+
+void i2c_init (int speed,int slaveaddr)
+{
+	// 	i2c_set_bus_speed (speed);
+	i2c_set_bus_num (CFG_CONTROLLER_BUS_NO);
+}
+
+int i2c_read(uchar chip, uint addr, int alen, uchar *buffer, int len)
+{
+	uchar tmp;
+	int i;
+	
+	if (len < 1 || alen < 0) {
+		return -1;
+	}
+
+	/* if shared bus */
+	if (i2c_bus == 2) {
+
+		/* request shared bus */
+		if (i2c_request_shared_bus(shared_bus_seg) != STATUS_SUCCESS) {
+			return -1;
+		}	
+		
+		/* wait for bus grant and if bus granted */
+		if (i2c_grant_shared_bus(shared_bus_seg) != STATUS_SUCCESS) {
+			return -1;
+		}
+		
+		/* start short timer */
+		fpga_start_short_timer();
+	}
+	
+	/* Wait for bus to stop */
+	WAIT_UNTIL((iread(SIC) & SIC_BBN) == SIC_BBN, I2C_TIMEOUT) {
+		debug("I2C bus timeout, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+		I2C_ABORT();
+		/* if shared bus */
+		if (i2c_bus == 2) {
+			/* release shared bus */
+			i2c_release_shared_bus(shared_bus_seg);
+		}		
+		return -1;
+	}
+
+	/* Send start */
+	iwrite(SID, SID_DATA((chip << 1) | ((alen == 0) ? 1 : 0)));
+	iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_STA | SIC_ACK);
+
+	/* Wait for NAK */
+	WAIT_UNTIL(((tmp = iread(SIC)) & SIC_PIN) == 0, I2C_TIMEOUT) {
+		I2C_STAT(tmp);
+		debug("I2C address timeout, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+		I2C_ABORT();
+		/* if shared bus */
+		if (i2c_bus == 2) {
+			/* release shared bus */
+			i2c_release_shared_bus(shared_bus_seg);
+		}		
+		
+		return -1;
+	}
+
+	if (tmp & SIC_LRB) {
+		debug("I2C slave NACK, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+		iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_STO | SIC_ACK);
+		/* if shared bus */
+		if (i2c_bus == 2) {
+			/* release shared bus */
+			i2c_release_shared_bus(shared_bus_seg);
+		}		
+		return -1;
+	}
+
+	/* Send address */
+	for (i = 0; i < alen; i++) {
+		I2C_STAT(tmp);
+		if ((tmp & SIC_LRB) != 0) {
+			debug("I2C slave missing, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+			I2C_ABORT();
+			/* if shared bus */
+			if (i2c_bus == 2) {
+				/* release shared bus */
+				i2c_release_shared_bus(shared_bus_seg);
+			}		
+			return -1;
+		}
+
+		iwrite(SID, SID_DATA(addr >> ((alen - i - 1) * 8)));
+		WAIT_UNTIL(((tmp = iread(SIC)) & SIC_PIN) == 0, I2C_TIMEOUT) {
+			I2C_STAT(tmp);
+			debug("I2C address timeout, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+			I2C_ABORT();
+			/* if shared bus */
+			if (i2c_bus == 2) {
+				/* release shared bus */
+				i2c_release_shared_bus(shared_bus_seg);
+			}			
+			return -1;
+		}
+	}
+
+	if (alen > 0) {
+		/* Send chip id */
+		iwrite(SID, SID_DATA((chip << 1) | 1));
+		/* Send repeated start */
+		iwrite(SIC, SIC_ES_DATA | SIC_STA | SIC_ACK);
+	}
+
+
+	for (i = 0; i < len; i++) {
+		buffer[i] = 0x80 + i;
+	}
+	for (i = -1; i < (len - 1); i++) {
+		WAIT_UNTIL((iread(SIC) & SIC_PIN) == 0, I2C_TIMEOUT) {
+			debug("I2C read timeout, bus = %d, chip = 0x%02x, offset %d\n", i2c_bus, chip, i);
+			I2C_ABORT();
+			/* if shared bus */
+			if (i2c_bus == 2) {
+				/* release shared bus */
+				i2c_release_shared_bus(shared_bus_seg);
+			}				
+			return -1;
+		}
+
+		if ((iread(SIC) & SIC_LRB) != 0) {
+			/* Error! */
+			iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_STO | SIC_ACK);
+			debug("I2C read error, bus = %d, chip = 0x%02x, offset %d\n", i2c_bus, chip, i);
+			tmp = iread(SID);	/* Throw-away data */
+			I2C_ABORT();
+			/* if shared bus */
+			if (i2c_bus == 2) {
+				/* release shared bus */
+				i2c_release_shared_bus(shared_bus_seg);
+			}				
+			return -1;
+		}
+
+		tmp = SID_DATA(iread(SID));
+		if (i >= 0) {
+			buffer[i] = tmp;
+		}
+	}
+
+	iwrite(SIC, SIC_ES_DATA);
+	tmp = SID_DATA(iread(SID));
+	if (i >= 0) {
+		buffer[i] = tmp;
+	}
+
+	WAIT_UNTIL((iread(SIC) & SIC_PIN) == 0, I2C_TIMEOUT) {
+		debug("I2C finish error, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+		I2C_ABORT();
+		/* if shared bus */
+		if (i2c_bus == 2) {
+			/* release shared bus */
+			i2c_release_shared_bus(shared_bus_seg);
+		}			
+		return -1;
+	}
+
+	iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_STO | SIC_ACK);
+	tmp = SID_DATA(iread(SID));
+	buffer[i] = tmp;
+
+	iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_ACK);
+
+	/* if shared bus */
+	if (i2c_bus == 2) {
+		/* release shared bus */
+		i2c_release_shared_bus(shared_bus_seg);
+	}
+	
+	return 0;
+}
+
+int i2c_write(uchar chip, uint addr, int alen, uchar * buffer, int len)
+{
+	int i;
+	uchar tmp;
+
+	/* if shared bus */
+	if (i2c_bus == 2) {
+
+		/* request shared bus */
+		if (i2c_request_shared_bus(shared_bus_seg) != STATUS_SUCCESS) {
+			return -1;
+		}	
+		
+		/* wait for bus grant and if bus granted */
+		if (i2c_grant_shared_bus(shared_bus_seg) != STATUS_SUCCESS) {
+			return -1;
+		}
+		
+		/* (re)start short timer */
+		fpga_start_short_timer();
+	}
+	
+	/* Wait for bus to stop */
+	WAIT_UNTIL((iread(SIC) & SIC_BBN) == SIC_BBN, I2C_TIMEOUT) {
+		debug("I2C bus timeout, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+		I2C_ABORT();
+		/* if shared bus */
+		if (i2c_bus == 2) {
+			/* release shared bus */
+			i2c_release_shared_bus(shared_bus_seg);
+		}
+		return -1;
+	}
+
+	/* Send start */
+	iwrite(SID, SID_DATA((chip << 1) | 0));
+	iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_STA | SIC_ACK);
+
+	/* Wait for NAK */
+	WAIT_UNTIL(((tmp = iread(SIC)) & SIC_PIN) == 0, I2C_TIMEOUT) {
+		I2C_STAT(tmp);
+		debug("I2C NAK timeout, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+		I2C_ABORT();
+		/* if shared bus */
+		if (i2c_bus == 2) {
+			/* release shared bus */
+			i2c_release_shared_bus(shared_bus_seg);
+		}
+		return -1;
+	}
+
+	if (tmp & SIC_LRB) {
+		debug("I2C slave NACK, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+		iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_STO | SIC_ACK);
+		/* if shared bus */
+		if (i2c_bus == 2) {
+			/* release shared bus */
+			i2c_release_shared_bus(shared_bus_seg);
+		}
+		return -1;
+	}
+
+	/* Send address */
+	for (i = 0; i < alen; i++) {
+		iwrite(SID, SID_DATA(addr >> ((alen - i - 1) * 8)));
+
+		/* (re)start short timer */
+		if (i2c_bus == 2) { fpga_start_short_timer(); }
+		
+		WAIT_UNTIL(((tmp = iread(SIC)) & SIC_PIN) == 0, I2C_TIMEOUT) {
+			I2C_STAT(tmp);
+			debug("I2C address timeout, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+			I2C_ABORT();
+			/* if shared bus */
+			if (i2c_bus == 2) {
+				/* release shared bus */
+				i2c_release_shared_bus(shared_bus_seg);
+			}
+			return -1;
+		}
+
+		if ((iread(SIC) & SIC_LRB) != 0) {
+			debug("I2C slave missing, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+			I2C_ABORT();
+			/* if shared bus */
+			if (i2c_bus == 2) {
+				/* release shared bus */
+				i2c_release_shared_bus(shared_bus_seg);
+			}
+			return -1;
+		}
+	}
+
+	/* Send data */
+	for (i = 0; i < len; i++) {
+		iwrite(SID, SID_DATA(buffer[i]));
+
+		/* (re)start short timer */
+		if (i2c_bus == 2) { fpga_start_short_timer(); }
+		
+		WAIT_UNTIL((iread(SIC) & SIC_PIN) == 0, I2C_TIMEOUT) {
+			debug("I2C address timeout, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+			I2C_ABORT();
+			/* if shared bus */
+			if (i2c_bus == 2) {
+				/* release shared bus */
+				i2c_release_shared_bus(shared_bus_seg);
+			}
+			return -1;
+		}
+
+		if ((iread(SIC) & SIC_LRB) != 0) {
+			debug("I2C slave missing, bus = %d, chip = 0x%02x\n", i2c_bus, chip);
+			I2C_ABORT();
+			/* if shared bus */
+			if (i2c_bus == 2) {
+				/* release shared bus */
+				i2c_release_shared_bus(shared_bus_seg);
+			}
+			return -1;
+		}
+	}
+
+	/* Send stop */
+	iwrite(SIC, SIC_PIN | SIC_ES_DATA | SIC_STO | SIC_ACK);
+
+	/* if shared bus */
+	if (i2c_bus == 2) {
+		/* release shared bus */
+		i2c_release_shared_bus(shared_bus_seg);
+	}
+	
+	return 0;
+}
+
+/*-------------------------------------------------------------------------
+	Copyright (C) 2008-20013 American Megatrends Inc.
+
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
+	
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+-------------------------------------------------------------------------*/
+
+int supfpga_mm_reg8_read(uint16_t reg, uchar *read_buf);
+int supfpga_mm_reg8_write(uint16_t reg, uchar write_buf);
+
+int i2c_request_shared_bus(uchar segnum);
+int i2c_grant_shared_bus(uchar segnum);
+int i2c_release_shared_bus(uchar segnum);
+
+int fpga_start_short_timer(void);
+int fpga_set_local_short_timeout(uchar timeout);
+int fpga_set_local_long_timeout(uchar timeout);
+
+//#define DEBUG
+//#define DEBUG_VERBOSE
+
+//#define DEBUG_SFPGA
+#define AVOID_ERROR_LOG_SPAM
+
+#ifdef DEBUG_SFPGA
+void dump_support_fpga_regs(void);
+#endif
+
+int
+supfpga_mm_reg8_read(uint16_t reg, uchar *read_buf) {
+	
+	*read_buf = (uchar)READ_SUPFPGA_MMAP_REG(reg);
+	return STATUS_SUCCESS;
+}
+
+int
+supfpga_mm_reg16_read(uint16_t reg, uint16_t *read_buf) {
+	
+	*read_buf = (uint16_t)READ_SUPFPGA_MMAP_REG(reg);
+	return STATUS_SUCCESS;
+}
+
+int
+supfpga_mm_reg8_write(uint16_t reg, uchar write_buf) {
+
+	WRITE_SUPFPGA_MMAP_REG(reg, (uint16_t)write_buf);
+	return STATUS_SUCCESS;
+}
+
+int 
+i2c_request_shared_bus(uchar segnum) {
+
+	uchar val_mlbr, val_mrbr, val_mler, val_mrer;
+	int retval=STATUS_SUCCESS;
+
+#ifdef DEBUG_VERBOSE
+	debug("i2c_request_shared_bus() ENTER: segnum=%d\n", (int)segnum);
+#endif
+
+	/* set local long timeout value */
+	fpga_set_local_long_timeout(4); 
+	/* set short timer */
+	fpga_set_local_short_timeout(255);
+	/* start short timer */
+	fpga_start_short_timer();
+
+	/* read local bus register (MLBR) */
+	supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &val_mlbr);
+
+	/* read remote bus register (MRBR) for debugging */
+	supfpga_mm_reg8_read(SUPFPGA_MRBR_REG, &val_mrbr);
+
+	/* if local bus request already asserted (MLBR[4] set) */
+	if (val_mlbr & MLBR_LOC_BUS_REQ) {
+		/* log warning */
+		debug("  i2c_request_shared_bus(): Local bus request already asserted, segnum=%d\n", (int)segnum);
+
+		/* clear local bus request */
+		supfpga_mm_reg8_write(SUPFPGA_MLBR_REG, (val_mlbr & ~MLBR_LOC_BUS_REQ));
+
+		/* again read local bus request */
+		supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &val_mlbr);
+
+		/* if local bus request is still asserted (MLBR[4] set) */
+		if (val_mlbr & MLBR_LOC_BUS_REQ) {
+			/* log error and return */
+			debug("i2c_request_shared_bus() FAIL: Local bus request still asserted after deasserting, segnum=%d, MLBR=0x%02x\n", 
+				(int)segnum, val_mlbr);
+			return STATUS_INACCESSIBLE;
+		} 
+	}
+
+	/* read local error register (MLER) */
+	supfpga_mm_reg8_read(SUPFPGA_MLER_REG, &val_mler);
+	/* read remote error register (MRER) */
+	supfpga_mm_reg8_read(SUPFPGA_MRER_REG, &val_mrer);
+	
+	/* if any local or remote errors */
+	if ((val_mler & CLEARABLE_ERROR_MASK) || (val_mrer & CLEARABLE_ERROR_MASK)) {
+		/* log errors */
+		debug("  i2c_request_shared_bus(): Local and/or remote errors detected... attempting to clear, segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", 
+			(int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+
+		/* if any local errors */
+		if (val_mler & CLEARABLE_ERROR_MASK) {
+			/* clear errors from local error register (note: clear is done be setting bit) */
+			supfpga_mm_reg8_write(SUPFPGA_MLER_REG, val_mler);
+			/* reread local error register (MLER) */
+			supfpga_mm_reg8_read(SUPFPGA_MLER_REG, &val_mler);
+		}
+
+		/* if any remote errors */
+		if (val_mrer & CLEARABLE_ERROR_MASK) {
+			/* clear errors from remote error register (note: clear is done be setting bit) */
+			supfpga_mm_reg8_write(SUPFPGA_MRER_REG, val_mrer);
+			/* reread remote error register (MRER) */
+			supfpga_mm_reg8_read(SUPFPGA_MRER_REG, &val_mrer);
+		}
+			
+		/* if we still have local or remote errors after clearing */
+		if ((val_mler != NO_ERROR) || (val_mrer != NO_ERROR)) {
+			debug("i2c_request_shared_bus() FAIL: Local and/or remote errors detected after clearing, segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", 
+				(int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+
+			return STATUS_INACCESSIBLE;
+		}
+	}
+
+	/* if local bus already active (MLBR[5]set) */
+	if (val_mlbr & MLBR_LOC_BUS_ACT) {
+		debug("i2c_request_shared_bus() FAIL: Local bus already active, segnum=%d, MLBR=0x%02x, MRBR=0x%02x\n", 
+			(int)segnum, val_mlbr, val_mrbr);
+		return STATUS_INACCESSIBLE;
+	} 
+
+	/* set request bus number (segment number) */
+	/* set the access type to read/write */
+	supfpga_mm_reg8_write(SUPFPGA_MLBR_REG, (segnum | MLBR_LOC_ACC_TYPE));
+	supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &val_mlbr);
+
+	/* request bus */
+	supfpga_mm_reg8_write(SUPFPGA_MLBR_REG, (MLBR_LOC_BUS_REQ | val_mlbr));
+
+	if (retval == STATUS_SUCCESS) {
+#ifdef DEBUG_VERBOSE	
+		/* read local bus register (MLBR) */
+		supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &val_mlbr);
+		debug("i2c_request_shared_bus() SUCCESS: segnum=%d, MLBR=0x%02x, MRBR=0x%02x\n", 
+			(int)segnum, val_mlbr, val_mrbr);
+#endif
+	}
+	return retval;
+}
+
+int 
+i2c_grant_shared_bus(u8 segnum) {
+
+	u8 val_mlbr, val_mrbr, val_mler, val_mrer, long_timeout;
+	int retval=STATUS_SUCCESS;
+	int time_left;
+	static u8 last_mler=0, last_mrer=0;
+	int timeout=0;
+	int log_error=1; 
+
+	/* read local bus register (MLBR) */
+	supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &val_mlbr);
+
+	/* read remote bus register (MRBR) */
+	supfpga_mm_reg8_read(SUPFPGA_MRBR_REG, &val_mrbr);
+
+	/* read long timeout value */
+	supfpga_mm_reg8_read(SUPFPGA_MLTR_REG, &long_timeout);
+
+	/* timeout in ms */	
+	time_left = (long_timeout * 1000);
+	
+#ifdef DEBUG_VERBOSE	
+	debug(KERN_INFO "i2c_grant_shared_bus() ENTER: segnum=%d\n", (int)segnum);
+#endif
+
+	/* while bus is not active (MLBR[5] clear) */
+	while ((0 == (val_mlbr & MLBR_LOC_BUS_ACT))) {
+
+		/* if timeout */
+		if (time_left <= 0) {
+			timeout = 1;
+			break;
+		}
+
+		/* pet short timer */
+		fpga_start_short_timer();
+		
+		/* Read local errors(MLER) and remote errors(MRER) */
+		supfpga_mm_reg8_read(SUPFPGA_MLER_REG, &val_mler);
+		supfpga_mm_reg8_read(SUPFPGA_MRER_REG, &val_mrer);
+		
+		/* if any error detected (local, remote, clearable, unclearable) */
+		if ((val_mler != NO_ERROR) || (val_mrer != NO_ERROR)) {
+
+#ifdef AVOID_ERROR_LOG_SPAM
+			/* if un-clearable errors only (invalid serial exchange bus AND local buffers not ready)*/
+			if (((val_mler & CLEARABLE_ERROR_MASK) == NO_ERROR) || ((val_mrer & CLEARABLE_ERROR_MASK) == NO_ERROR)) {
+
+				 /* Some errors are not clearable and can potentially cause log spam.  
+				 Avoid log spam by only logging once everytime error asserts */
+				if ( /*((val_mler & 0x80) && (last_mler & 0x80)) || RESERVED */ 
+					 ((val_mrer & 0x80) && (last_mrer & 0x80)) ||
+					 ((val_mler & 0x40) && (last_mler & 0x40)) ||
+					 ((val_mrer & 0x40) && (last_mrer & 0x40)) ||
+					 ((val_mler & 0x20) && (last_mler & 0x20)) ||
+					 ((val_mrer & 0x20) && (last_mrer & 0x20)) ) {
+						log_error = 0; /* this error has already been logged, so skip logging */
+				}
+			}
+#endif
+			/* log the error (to avoid log spam from un-clearable errors 
+			we only log if error has changed since last loop) */
+			if (log_error) {
+				debug("  i2c_grant_shared_bus(): Local and/or remote errors... attempting to clear, segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", 
+					(int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+			}
+
+#ifdef DEBUG_SFPGA
+			dump_support_fpga_regs();
+			retval = STATUS_INACCESSIBLE;
+			break; /* note: request will be released before we return from function */
+#endif
+			
+			/* release bus before attempting to clear errors */
+			i2c_release_shared_bus(segnum);
+
+			/* if mler errors attempt to clear */
+			if ((val_mler & CLEARABLE_ERROR_MASK) != NO_ERROR) {
+				supfpga_mm_reg8_write(SUPFPGA_MLER_REG, val_mler);
+				supfpga_mm_reg8_read(SUPFPGA_MLER_REG, &val_mler);
+			}
+			/* if mrer errors attempt to clear */
+			if ((val_mrer & CLEARABLE_ERROR_MASK) != NO_ERROR) {
+				supfpga_mm_reg8_write(SUPFPGA_MRER_REG, val_mrer);
+				supfpga_mm_reg8_read(SUPFPGA_MRER_REG, &val_mrer);
+			}
+
+			/* re-request bus request */
+			if (i2c_request_shared_bus(segnum) != 0) {
+				debug("  i2c_grant_shared_bus(): Re-request of bus after error detection failed, segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", 
+					(int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+				retval = STATUS_INACCESSIBLE;
+				break;
+			}
+
+			/* if still have errors... log error */
+			if (((val_mler & CLEARABLE_ERROR_MASK) != NO_ERROR) || ((val_mrer & CLEARABLE_ERROR_MASK) != NO_ERROR)) {
+			
+				debug(KERN_WARNING "  i2c_grant_shared_bus(): Local and/or remote errors after clear, segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", 
+					(int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+
+				retval = STATUS_INACCESSIBLE;
+				break;
+			}
+		}
+
+		last_mler = val_mler;
+		last_mrer = val_mrer;
+
+		/* wait a little bit before trying again */
+		udelay((time_left > 1000) ? 1000 : time_left);
+		time_left -= (time_left > 1000)  ? 1000 : time_left;
+
+		/* re-read MLBR and MRBR registers (update val_mlbr and val_mrbr) */
+		supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &val_mlbr);
+		supfpga_mm_reg8_read(SUPFPGA_MRBR_REG, &val_mrbr);
+	}
+ 
+	/* if wait timer >= long timeout value */
+	if (timeout) {
+#ifdef DEBUG
+		debug("  i2c_grant_shared_bus(): Timeout waiting for bus grant, segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", 
+			(int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+#endif	
+		retval = STATUS_INACCESSIBLE;
+	}
+
+	supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &val_mlbr);
+	supfpga_mm_reg8_read(SUPFPGA_MRBR_REG, &val_mrbr);
+	supfpga_mm_reg8_read(SUPFPGA_MLER_REG, &val_mler);
+	supfpga_mm_reg8_read(SUPFPGA_MRER_REG, &val_mrer);
+
+	/* release bus request on failures */
+	if (retval != STATUS_SUCCESS) {
+#ifdef DEBUG_VERBOSE	
+		debug(KERN_WARNING "i2c_grant_shared_bus() FAIL, segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", 
+			(int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+#endif			
+		i2c_release_shared_bus(segnum);
+	}
+
+	if (retval == STATUS_SUCCESS) {
+		fpga_start_short_timer();
+#ifdef DEBUG_VERBOSE	
+		debug(KERN_INFO "i2c_grant_shared_bus() SUCCESS, segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", 
+			(int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+#endif	
+	} 
+	
+	return retval;
+}
+
+int
+i2c_release_shared_bus(u8 segnum) {
+
+	u8 val_mlbr;
+#ifdef DEBUG_VERBOSE	
+	u8 val_mrbr, val_mler, val_mrer;
+#endif
+
+	/* read local bus register */
+	supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &val_mlbr);
+
+	/* release bus request (clear MLBR[4]) */
+	supfpga_mm_reg8_write(SUPFPGA_MLBR_REG, (val_mlbr & ~MLBR_LOC_BUS_REQ));	
+
+#ifdef DEBUG_VERBOSE
+	/* log any errors (we will clear them on next access) */
+	supfpga_mm_reg8_read(SUPFPGA_MRBR_REG, &val_mrbr);
+	supfpga_mm_reg8_read(SUPFPGA_MLER_REG, &val_mler);
+	supfpga_mm_reg8_read(SUPFPGA_MRER_REG, &val_mrer);
+	if (((val_mler & CLEARABLE_ERROR_MASK)!= NO_ERROR) || ((val_mrer & CLEARABLE_ERROR_MASK) != NO_ERROR)) {
+		debug("i2c_release_shared_bus(): segnum=%d, MLBR=0x%02x, MRBR=0x%02x, MLER=0x%02x, MRER=0x%02x\n", (int)segnum, val_mlbr, val_mrbr, val_mler, val_mrer);
+	}
+#endif
+
+	return STATUS_SUCCESS;
+}
+
+
+int 
+fpga_start_short_timer() {
+
+	static int cnt=0;
+	(cnt == 2) ? cnt = 0 : cnt++;
+
+	supfpga_mm_reg8_write(SUPFPGA_MTRR_REG, MTRR_START_STIMER);
+
+	return STATUS_SUCCESS;
+}
+
+int 
+fpga_set_local_short_timeout(u8 timeout) {
+	
+#ifdef DEBUG_VERBOSE	
+	debug("fpga set local short timeout to %d millisecond(s)\n", (int)timeout);
+#endif
+	supfpga_mm_reg8_write(SUPFPGA_MSTR_REG, timeout);
+
+	return STATUS_SUCCESS;
+}
+
+int
+fpga_set_local_long_timeout(u8 timeout) {
+	
+	/* max allowed long timeout value is 15 */
+	if (timeout > 15) {
+#ifdef DEBUG_VERBOSE	
+		debug("fpga set local long timeout to %d second(s) exceeds max of 15 seconds\n   forcing timeout to 15 seconds\n", (int)timeout);
+#endif
+		timeout = 15;
+	}
+	
+#ifdef DEBUG_VERBOSE	
+	debug("fpga set local long timeout to %d second(s)\n", (int)timeout);
+#endif
+	supfpga_mm_reg8_write(SUPFPGA_MLTR_REG, timeout);
+
+	return STATUS_SUCCESS;
+}
+
+#ifdef DEBUG_SFPGA
+void
+dump_support_fpga_regs() {
+
+	uint16_t val, usw;
+
+	/* log support fpga registers */
+	debug("  SUPPORT FPGA REG DUMP: \n");
+
+	supfpga_mm_reg16_read(SUPFPGA_MCSR_REG, &val);
+	debug("    MCSR=0x%04x\n", val);
+
+	supfpga_mm_reg16_read(SUPFPGA_MLBR_REG, &val);
+	debug("    MLBR=0x%04x\n", val);
+
+	supfpga_mm_reg16_read(SUPFPGA_MTRR_REG, &val);
+	debug("    MTRR=0x%04x\n", val);
+
+	supfpga_mm_reg16_read(SUPFPGA_MRBR_REG, &val);
+	debug("    MRBR=0x%04x\n", val);
+
+	supfpga_mm_reg16_read(SUPFPGA_MLER_REG, &val);
+	debug("    MLER=0x%04x\n", val);
+
+	supfpga_mm_reg16_read(SUPFPGA_MRER_REG, &val);
+	debug("    MRER=0x%04x\n", val);
+
+	supfpga_mm_reg16_read(SUPFPGA_MSTR_REG, &val);
+	debug("    MSTR=0x%04x\n", val);
+
+	supfpga_mm_reg16_read(SUPFPGA_MLTR_REG, &val);
+	debug("    MLTR=0x%04x\n", val);
+
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG0_REG, &val);
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG0_REG-2, &usw);
+	debug("    C000=0x%04x%04x\n", usw, val);
+
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG1_REG, &val);
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG1_REG-2, &usw);
+	debug("    C004=0x%04x%04x\n", usw, val);
+
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG2_REG, &val);
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG2_REG-2, &usw);
+	debug("    C008=0x%04x%04x\n", usw, val);
+
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG3_REG, &val);
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG3_REG-2, &usw);
+	debug("    C00C=0x%04x%04x\n", usw, val);
+
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG4_REG, &val);
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG4_REG-2, &usw);
+	debug("    C010=0x%04x%04x\n", usw, val);
+
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG5_REG, &val);
+	supfpga_mm_reg16_read(SUPFPGA_DEBUG5_REG-2, &usw);
+	debug("    C014=0x%04x%04x\n", usw, val);
+	
+	return;
+}			
+#endif
+
diff -Naur uboot.old/include/configs/VSC452EVB.h uboot/include/configs/VSC452EVB.h
--- uboot.old/include/configs/VSC452EVB.h	2008-10-16 10:45:21.000000000 -0400
+++ uboot/include/configs/VSC452EVB.h	2008-10-16 11:02:05.000000000 -0400
@@ -76,7 +76,7 @@
  *---------------------------------------------------------------
  */
 #define CONFIG_COMMANDS		\
-	(CFG_CMD_DHCP|CFG_CMD_NET| CFG_CMD_PING|CFG_CMD_MII|	\
+	(CFG_CMD_EEPROM|CFG_CMD_DHCP|CFG_CMD_NET| CFG_CMD_PING|CFG_CMD_MII|	\
 	CFG_CMD_MEMORY|CFG_CMD_CONSOLE|CFG_CMD_FLASH|		\
 	CFG_CMD_AUTOSCRIPT|CFG_CMD_RUN|CFG_CMD_BDI|		\
 	CFG_CMD_ELF|CFG_CMD_IMLS|CFG_CMD_IMI|			\
@@ -164,6 +164,17 @@
 #define NAND_MAX_CHIPS		1
 
 
+/*---------------------------------------------------------------
+ * EEPROM 
+ *----------------------------------------------------------------
+ */
+#define CFG_I2C_EEPROM_ADDR     0x50
+#define CFG_I2C_EEPROM_ADDR_LEN 2
+#define CONFIG_HARD_I2C         1
+#define CFG_I2C_SLAVE           CFG_I2C_EEPROM_ADDR
+#define CFG_I2C_SPEED           100000
+
+
 /*----------------------------------------------------------------
  *             Physical Memory Map of SDRAM and Flash
  *----------------------------------------------------------------
diff -Naur uboot.old/include/vsc452/vsc452_i2c.h uboot/include/vsc452/vsc452_i2c.h
--- uboot.old/include/vsc452/vsc452_i2c.h	1969-12-31 19:00:00.000000000 -0500
+++ uboot/include/vsc452/vsc452_i2c.h	2013-10-14 14:11:20.708406000 -0400
@@ -0,0 +1,191 @@
+/*
+ * $Id$
+ *
+ * Copyright 2008, Network Appliance Inc.
+ * Author: Jason McMullan <mcmullan@netapp.com>
+ *
+ */
+
+#ifndef ASM_MIPS_ARCH_VSC452_I2C_H
+#define ASM_MIPS_ARCH_VSC452_I2C_H
+
+#define ARRAY_SIZE(x) (sizeof(x) /sizeof((x)[0]))
+
+#define VSC452_I2C_INTERFACES	6
+#define VSC452_I2C_FIFO_MAX	80	/* 80 byte FIFO */
+
+#define I2C_BASE(x)	(0x10200500 + (0x80 * (x)))
+
+#define SID		0x00	/* Serial Interface Data */
+#define SID_DATA(x)		((x) & 0xff)	/* Data */
+#define SID_SCA_MASTER		(1 << 7)	/* Master-only interface */
+#define SID_SCA_ADDR(x)		((x) & 0x7f)	/* Slave address of interface */
+#define SID_SSCA_ENABLE		(1 << 7)	/* Enable SSCA */
+#define SID_SSCA_ADDR(x)	((x) & 0x7f)	/* Secondary Slave address of interface */
+#define SID_CMS_NAS		(1 << 7)	/* Don't snoop the bus */
+#define SID_CMS_CLK_98KHZ	(0 << 0)	/* Clock Select */
+#define SID_CMS_CLK_78KHZ	(1 << 0)	/* Clock Select */
+#define SID_CMS_CLK_347KHZ	(2 << 0)	/* Clock Select */
+#define SID_CMS_CLK_284KHZ	(3 << 0)	/* Clock Select */
+#define SID_CMS_CLK_417KHZ	(4 << 0)	/* Clock Select */
+#define SID_CMS_CLK_340KHZ	(5 << 0)	/* Clock Select */
+#define SID_CMS_CLK_1042KHZ	(6 << 0)	/* Clock Select */
+#define SID_CMS_CLK_852KHZ	(7 << 0)	/* Clock Select */
+
+#define SIC		0x04	/* Serial Interface Control */
+	/* Read Mode */
+#define SIC_PIN		(1 << 7)		/* Pending Interrupt Not */
+#define SIC_SSN		(1 << 6)		/* Start Status Not */
+#define SIC_STS		(1 << 5)		/* Stop Status */
+#define SIC_BER		(1 << 4)		/* Bus Error */
+#define SIC_LRB		(1 << 3)		/* Last Received Bit */
+#define SIC_AAS		(1 << 2)		/* Addressed as Slave */
+#define SIC_LAB		(1 << 1)		/* Lost Arbitaration */
+#define SIC_BBN		(1 << 0)		/* Bus Busy Not */
+	/* Write Mode */
+#define SIC_ES_SCA	(0 << 4)		/* SID is Slave Compare Address */
+#define SIC_ES_SSCA	(7 << 4)		/* SID is Secondary Slave Compare Address */
+#define SIC_ES_DTH	(1 << 4)		/* SID is Data Out Temporary Holding Reg */
+#define SIC_ES_CMS	(2 << 4)		/* SID is Clock and Mode selection */
+#define SIC_ES_DATA	(4 << 4)		/* SID is 8-byte data in/out */
+#define SIC_ENI		(1 << 3)		/* Enable Interrupt */
+#define SIC_STA		(1 << 2)		/* Send Start */
+#define SIC_STO		(1 << 1)		/* Send Stop */
+#define SIC_ACK		(1 << 0)		/* Acknowledge */
+
+#define SIBC		0x08	/* Serial Interface Bit Control */
+#define SIBC_SISR	(1 << 7)		/* Soft Reset */
+#define SIBC_SDIE	(1 << 6)		/* Stop Detected Interrupt Enable */
+#define SIBC_SCC	(1 << 1)		/* Serial Clock Control */
+#define SIBC_SDC	(1 << 0)		/* Serial Data Control */
+
+#define SIBS		0x0c	/* Serial Interface Bit Status */
+#define SIBS_SIIS	(1 << 7)		/* Serial Interface Interrupt Status */
+#define SIBS_SDIS	(1 << 6)		/* Stop Detected Interrupt Status */
+#define SIBS_SCS	(1 << 1)		/* Serial Clock Status */
+#define SIBS_SDS	(1 << 0)		/* Serial Data Status */
+
+#define SIFC		0x10	/* Serial Interface RX FIFO Control */
+#define SIFC_SRFE	(1 << 7)		/* Slave RX FIFO Enable */
+#define SIFC_CRFP	(1 << 6)		/* Clear RX FIFO Pointers */
+#define SIFC_STMIE	(1 << 1)		/* Slave TX Mode Interrupt Enable */
+#define SIFC_RFIE	(1 << 0)		/* RX FIFO Interrupt Enable */
+
+#define SIFS		0x14	/* Serial Interface RX FIFO Status */
+#define SIFS_RFF	(1 << 3)		/* RX FIFO Full */
+#define SIFS_RFE	(1 << 2)		/* RX FIFO Empty */
+#define SIFS_STMIS	(1 << 1)		/* Slave Transmit Mode Interrupt Status */
+#define SIFS_RFIS	(1 << 0)		/* Receive FIFO Interrupt Status */
+
+#define SIFD		0x18	/* Serial Interface RX FIFO Data */
+#define SIFD_FD(x)	((x) & 0xff)		/* FIFO Data */
+
+/*-------------------------------------------------------------------------
+	Copyright (C) 2008-20013 American Megatrends Inc.
+	
+	This program is free software; you can redistribute it and/or modify
+	it under the terms of the GNU General Public License as published by
+	the Free Software Foundation; either version 2 of the License, or
+	(at your option) any later version.
+	
+	This program is distributed in the hope that it will be useful,
+	but WITHOUT ANY WARRANTY; without even the implied warranty of
+	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+	GNU General Public License for more details.
+	
+	You should have received a copy of the GNU General Public License
+	along with this program; if not, write to the Free Software
+	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+-------------------------------------------------------------------------*/
+
+#define STATUS_SUCCESS                          0
+#define STATUS_INVALID_PARAM                    -1
+#define STATUS_INACCESSIBLE                     -2
+#define STATUS_OPER_INCOMPLETE                  -3
+#define STATUS_FAILURE                          -4
+
+#define NO_ERROR                                0x00
+#define CLEARABLE_ERROR_MASK                    0x1f
+
+ /* PARALLEL_BUS_MLBR */
+#define MLBR_LOC_BUS_NUM						0x07
+#define MLBR_LOC_BUS_REQ						0x10
+#define MLBR_LOC_BUS_ACT						0x20
+#define MLBR_LOC_ACC_TYPE						0x40
+
+/* PARALLEL_BUS_MTRR */
+#define MTRR_REST_SRT_TIMER						0x01
+
+/* PARALLEL_BUS_MRBR */
+#define MRBR_RM_BUS_NUM							0x03
+#define MRBR_RM_BUS_GRT							0x10
+#define MRBR_RM_BUS_ACT							0x20
+#define MRBR_RM_ACC_TYPE						0x40
+
+/* PARALLEL_BUS_MLER */
+#define MLER_LOC_REL_SHORT_TMO					0x01
+#define MLER_LOC_REL_LONG_TMO					0x02
+#define MLER_RM_GRT_TMO							0x04
+#define MLER_RM_REL_TMO							0x08
+#define MLER_LOC_SER_EXCH_ERR					0x10
+#define MLER_LOC_INV_SER_EXCH_BUS				0x20
+#define MLER_LOC_BUF_N_READY					0x40
+
+/* PARALLEL_BUS_MTRR */
+#define MTRR_START_STIMER						0x01
+
+/* PARALLEL_BUS_MRER */
+#define MRER_RM_REL_SHORT_TMO					0x01
+#define MRER_RM_REL_LONG_TMO					0x02
+#define MRER_LOC_GRT_TMO						0x04
+#define MRER_LOC_REL_TMO						0x08
+#define MRER_RM_SER_EXCH_ERR					0x10
+#define MRER_RM_INV_SER_EXCH_BUS				0x20
+#define MRER_RM_BUF_N_READY						0x40
+
+#define SUPFPGA_BASE             0xB6000000
+#define SUPFPGA_MCSR_REG         0x1002
+#define SUPFPGA_MLBR_REG         0x1006
+#define SUPFPGA_MTRR_REG         0x100A
+#define SUPFPGA_MRBR_REG         0x100E
+#define SUPFPGA_MLER_REG         0x1012
+#define SUPFPGA_MRER_REG         0x1016
+#define SUPFPGA_MSTR_REG         0x1022
+#define SUPFPGA_MLTR_REG         0x1026
+
+/* for support fpga debugging */
+#define SUPFPGA_DEBUG0_REG       0xC002
+#define SUPFPGA_DEBUG1_REG       0xC006
+#define SUPFPGA_DEBUG2_REG       0xC00A
+#define SUPFPGA_DEBUG3_REG       0xC00E
+#define SUPFPGA_DEBUG4_REG       0xC012
+#define SUPFPGA_DEBUG5_REG       0xC016
+
+#define READ_SUPFPGA_MMAP_REG(reg)        (*((uint16_t *)(SUPFPGA_BASE + reg)))
+#define WRITE_SUPFPGA_MMAP_REG(reg, val)  *((uint16_t *)(SUPFPGA_BASE + reg)) = (uint16_t)val
+
+#define RESTART_SHORTTIMER
+
+#define RESTART_TIMER_VALUE	         0x0001
+
+#define RESTART_SHORT_TIMER(val)                   \
+do {                                               \
+    uchar mlbr;                                       \
+    supfpga_mm_reg8_read(SUPFPGA_MLBR_REG, &mlbr); \
+    if (mlbr & MLBR_LOC_BUS_ACT) {                 \
+        fpga_start_short_timer();                  \
+    }                                              \
+} while(0);
+
+#define MAX_SEG_VAL                  7
+
+int i2c_request_shared_bus(uchar segnum);
+int i2c_grant_shared_bus(uchar segnum);
+int i2c_release_shared_bus(uchar segnum);
+
+int fpga_start_short_timer(void);
+int fpga_set_local_short_timeout(uchar timeout);
+int fpga_set_local_long_timeout(uchar timeout);
+
+#endif /* ASM_MIPS_ARCH_VSC452_I2C_H */
+
diff -Naur uboot.old/Makefile uboot/Makefile
--- uboot.old/Makefile	2008-10-16 10:45:19.000000000 -0400
+++ uboot/Makefile	2008-10-16 10:46:14.000000000 -0400
@@ -204,6 +204,7 @@
 LIBS += rtc/librtc.a
 LIBS += dtt/libdtt.a
 LIBS += drivers/spiflash/libspiflash.a
+LIBS += drivers/i2c/libi2c.a
 LIBS += drivers/libdrivers.a
 LIBS += drivers/nand/libnand.a
 LIBS += drivers/nand_legacy/libnand_legacy.a
